/*
 * Copyright (c) 2014 J. Lewis Muir <jlmuir@imca-cat.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package imcacat.jcbf;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertArrayEquals;

import org.junit.Test;

/**
 * I test <code>JavaUtilities</code>.
 */
public class JavaUtilitiesTest {
  @Test
  public void testToCodePoints_null() {
    assertNull(JavaUtilities.toCodePoints(null));
  }

  @Test
  public void testToCodePoints_empty() {
    assertArrayEquals(new int[0], JavaUtilities.toCodePoints(""));
  }

  @Test
  public void testToCodePoints_basic() {
    String s = "abc";
    assertEquals(3, s.length());
    assertArrayEquals(new int[] {0x61, 0x62, 0x63}, JavaUtilities.toCodePoints(s));
  }

  @Test
  public void testToCodePoints_suplementary() {
    int[] expected = new int[] {0x1D15D, 0x1D15E, 0x1D15F};
    String s = toString(expected);
    assertEquals(6, s.length());
    assertArrayEquals(expected, JavaUtilities.toCodePoints(s));
  }

  @Test
  public void testToCodePoints_basicAndSuplementary() {
    int[] expected = new int[] {0x1D15D, 0x61, 0x1D15E, 0x1D15F};
    String s = toString(expected);
    assertEquals(7, s.length());
    assertArrayEquals(expected, JavaUtilities.toCodePoints(s));
  }

  @Test
  public void testTrim_null() {
    int[] trimCharsCodePoints = new int[] {" ".codePointAt(0)};
    assertNull(JavaUtilities.trim(null, toString(trimCharsCodePoints)));
    assertNull(JavaUtilities.trim(null, trimCharsCodePoints));
  }

  @Test
  public void testTrim_empty() {
    int[] trimCharsCodePoints = new int[] {" ".codePointAt(0)};
    assertEquals("", JavaUtilities.trim("", toString(trimCharsCodePoints)));
    assertEquals("", JavaUtilities.trim("", trimCharsCodePoints));
  }

  @Test
  public void testTrim_nothingToTrim() {
    int[] trimCharsCodePoints = new int[] {" ".codePointAt(0)};
    assertEquals("abc", JavaUtilities.trim("abc", toString(trimCharsCodePoints)));
    assertEquals("abc", JavaUtilities.trim("abc", trimCharsCodePoints));
  }

  @Test
  public void testTrim_leadingSpaces() {
    int[] trimCharsCodePoints = new int[] {" ".codePointAt(0)};
    assertEquals("abc", JavaUtilities.trim("  abc", toString(trimCharsCodePoints)));
    assertEquals("abc", JavaUtilities.trim("  abc", trimCharsCodePoints));
  }

  @Test
  public void testTrim_trailingSpaces() {
    int[] trimCharsCodePoints = new int[] {" ".codePointAt(0)};
    assertEquals("abc", JavaUtilities.trim("abc  ", toString(trimCharsCodePoints)));
    assertEquals("abc", JavaUtilities.trim("abc  ", trimCharsCodePoints));
  }

  @Test
  public void testTrim_leadingAndTrailingSpaces() {
    int[] trimCharsCodePoints = new int[] {" ".codePointAt(0)};
    assertEquals("abc", JavaUtilities.trim("  abc  ", toString(trimCharsCodePoints)));
    assertEquals("abc", JavaUtilities.trim("  abc  ", trimCharsCodePoints));
  }

  @Test
  public void testTrim_leadingAndTrailingSpacesAndBangs() {
    int[] trimCharsCodePoints = new int[] {" ".codePointAt(0), "!".codePointAt(0)};
    assertEquals("abc", JavaUtilities.trim("! abc! ", toString(trimCharsCodePoints)));
    assertEquals("abc", JavaUtilities.trim("! abc! ", trimCharsCodePoints));
  }

  @Test
  public void testTrim_leadingSupplementary() {
    int[] sCodePoints = new int[] {0x1D15D, 0x61, 0x1D15E, 0x1D15F};
    int[] trimCharsCodePoints = new int[] {0x1D15D};
    int[] expectedCodePoints = new int[] {0x61, 0x1D15E, 0x1D15F};
    String s = toString(sCodePoints);
    String trimChars = toString(trimCharsCodePoints);
    String expected = toString(expectedCodePoints);
    assertEquals(expected, JavaUtilities.trim(s, trimChars));
    assertEquals(expected, JavaUtilities.trim(s, trimCharsCodePoints));
  }

  @Test
  public void testTrim_trailingSupplementary() {
    int[] sCodePoints = new int[] {0x1D15D, 0x61, 0x1D15E, 0x1D15F};
    int[] trimCharsCodePoints = new int[] {0x1D15F};
    int[] expectedCodePoints = new int[] {0x1D15D, 0x61, 0x1D15E};
    String s = toString(sCodePoints);
    String trimChars = toString(trimCharsCodePoints);
    String expected = toString(expectedCodePoints);
    assertEquals(expected, JavaUtilities.trim(s, trimChars));
    assertEquals(expected, JavaUtilities.trim(s, trimCharsCodePoints));
  }

  @Test
  public void testTrim_leadingAndTrailingSupplementary() {
    int[] sCodePoints = new int[] {0x1D15D, 0x61, 0x1D15E, 0x1D15F};
    int[] trimCharsCodePoints = new int[] {0x1D15D, 0x1D15F};
    int[] expectedCodePoints = new int[] {0x61, 0x1D15E};
    String s = toString(sCodePoints);
    String trimChars = toString(trimCharsCodePoints);
    String expected = toString(expectedCodePoints);
    assertEquals(expected, JavaUtilities.trim(s, trimChars));
    assertEquals(expected, JavaUtilities.trim(s, trimCharsCodePoints));
  }

  @Test
  public void testTrim_leadingAndTrailingSupplementaryMoreThanOne() {
    int[] sCodePoints = new int[] {0x1D15D, 0x1D15D, 0x61, 0x1D15E, 0x1D15F};
    int[] trimCharsCodePoints = new int[] {0x1D15D, 0x1D15E, 0x1D15F};
    int[] expectedCodePoints = new int[] {0x61};
    String s = toString(sCodePoints);
    String trimChars = toString(trimCharsCodePoints);
    String expected = toString(expectedCodePoints);
    assertEquals(expected, JavaUtilities.trim(s, trimChars));
    assertEquals(expected, JavaUtilities.trim(s, trimCharsCodePoints));
  }

  @Test
  public void testTrim_leadingBasicAndTrailingSupplementary() {
    int[] sCodePoints = new int[] {0x61, 0x1D15D, 0x1D15E, 0x1D15F};
    int[] trimCharsCodePoints = new int[] {0x61, 0x1D15F};
    int[] expectedCodePoints = new int[] {0x1D15D, 0x1D15E};
    String s = toString(sCodePoints);
    String trimChars = toString(trimCharsCodePoints);
    String expected = toString(expectedCodePoints);
    assertEquals(expected, JavaUtilities.trim(s, trimChars));
    assertEquals(expected, JavaUtilities.trim(s, trimCharsCodePoints));
  }

  @Test
  public void testTrim_untouchedTrimCharactersInside() {
    int[] sCodePoints = new int[] {0x61, 0x1D15D, 0x1D15E, 0x1D15F};
    int[] trimCharsCodePoints = new int[] {0x1D15D};
    int[] expectedCodePoints = sCodePoints;
    String s = toString(sCodePoints);
    String trimChars = toString(trimCharsCodePoints);
    String expected = toString(expectedCodePoints);
    assertEquals(expected, JavaUtilities.trim(s, trimChars));
    assertEquals(expected, JavaUtilities.trim(s, trimCharsCodePoints));
  }

  private static String toString(int[] codePoints) {
    return new String(codePoints, 0, codePoints.length);
  }
}
