/*
 * Copyright (c) 2014 J. Lewis Muir <jlmuir@imca-cat.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package imcacat.jcbf;

import java.util.ArrayList;
import java.util.Arrays;

/**
 * Static utility methods to augment the core Java library.
 */
class JavaUtilities {
  private JavaUtilities() {
    throw new AssertionError();
  }

  /**
   * Converts the specified string to an array of Unicode code points.
   *
   * @param s the string to convert
   *
   * @return a newly allocated array of Unicode code points, or {@code null} if
   *         the specified string is {@code null}
   */
  public static int[] toCodePoints(String s) {
    if (s == null) return null;
    ArrayList<Integer> codePoints = new ArrayList<Integer>(s.length());
    for (int each, i = 0; i < s.length(); i += Character.charCount(each)) {
      each = s.codePointAt(i);
      codePoints.add(each);
    }
    int[] result = new int[codePoints.size()];
    for (int i = 0; i < result.length; i++) {
      result[i] = codePoints.get(i);
    }
    return result;
  }

  /**
   * Returns a copy of the specified string with any of the specified trim
   * characters removed from the beginning and end.
   *
   * @param s the string to trim
   * @param trimChars a set of characters to trim
   *
   * @return a copy of the string with leading and trailing trim characters
   *         removed, or this string if it has no leading or trailing trim
   *         characters
   */
  public static String trim(String s, String trimChars) {
    if (s == null) return s;
    if (s.length() == 0) return s;
    int beginIndex = 0;
    for (int eachCharCount, i = 0; i < s.length(); i += eachCharCount) {
      int each = s.codePointAt(i);
      eachCharCount = Character.charCount(each);
      if (trimChars.indexOf(each) == -1) break;
      beginIndex += eachCharCount;
    }
    int endIndex = s.length();
    int minIndex = beginIndex + 2;
    for (int eachCharCount, i = s.length(); i >= minIndex; i -= eachCharCount) {
      int each = s.codePointBefore(i);
      eachCharCount = Character.charCount(each);
      if (trimChars.indexOf(each) == -1) break;
      endIndex -= eachCharCount;
    }
    if (beginIndex == 0 && endIndex == s.length()) return s;
    return s.substring(beginIndex, endIndex);
  }

  /**
   * Returns a copy of the specified string with any of the specified trim
   * characters removed from the beginning and end.
   *
   * @param s the string to trim
   * @param trimChars a sorted array of characters (Unicode code points) to
   *          trim
   *
   * @return a copy of the string with leading and trailing trim characters
   *         removed, or this string if it has no leading or trailing trim
   *         characters
   */
  public static String trim(String s, int[] trimChars) {
    if (s == null) return s;
    if (s.length() == 0) return s;
    int beginIndex = 0;
    for (int eachCharCount, i = 0; i < s.length(); i += eachCharCount) {
      int each = s.codePointAt(i);
      eachCharCount = Character.charCount(each);
      if (Arrays.binarySearch(trimChars, each) < 0) break;
      beginIndex += eachCharCount;
    }
    int endIndex = s.length();
    int minIndex = beginIndex + 2;
    for (int eachCharCount, i = s.length(); i >= minIndex; i -= eachCharCount) {
      int each = s.codePointBefore(i);
      eachCharCount = Character.charCount(each);
      if (Arrays.binarySearch(trimChars, each) < 0) break;
      endIndex -= eachCharCount;
    }
    if (beginIndex == 0 && endIndex == s.length()) return s;
    return s.substring(beginIndex, endIndex);
  }
}
